@@ -26,6 +26,7 @@ import inspect
 from ..utils import augtool, BuildMetadata, Fstab, bcolors
 from ..naming import Image
 from ..lvm import LVM
+from ..bootloader import BootConfiguration
 
 
 log = logging.getLogger(__package__)
@@ -460,4 +461,35 @@ class Health():
 
         return group
 
+    def check_bootloader(self):
+        group = Health.CheckGroup()
+        b = BootConfiguration()
+        group.description = "Bootloader"
+        group.reason = ("It looks like there are no valid bootloader "
+                        "entries. Please ensure this is fixed before "
+                        "rebooting.")
+
+        def check_node():
+            if not b.list():
+                return False
+            return True
+
+        def check_other():
+            if not b.list_other():
+                return False
+            return True
+
+        group.checks = [
+            Health.Check("Layer boot entries",
+                         check_node,
+                         lambda: ("No bootloader entries which point to "
+                                  "imgbased layers")),
+            Health.Check("Valid boot entries",
+                         lambda: check_node() or check_other(),
+                         lambda: ("No valid boot entries for imgbased layers "
+                                  "or non-imgbased layers"))
+            ]
+
+        return group
+
 # vim: sw=4 et sts=4:
