@@ -25,6 +25,7 @@ import logging
 import glob
 import os
 import shutil
+import subprocess
 from .. import bootloader, utils
 from ..lvm import LVM
 from ..naming import Image
@@ -109,8 +110,9 @@ def migrate_etc(imgbase, new_lv, previous_lv):
         log.info("Verifying stream compatability")
         log.debug("%s vs %s" % (old_rel, new_rel))
 
-        if new_rel.PRODUCT not in ["fedora", "centos"]:
-            log.error("Unsupported üproduct: %s" % new_rel)
+        if new_rel.PRODUCT not in ["fedora", "centos",
+                                   "enterprise_linux"]:
+            log.error("Unsupported product: %s" % new_rel)
 
         is_same_product = old_rel.PRODUCT == new_rel.PRODUCT
 
@@ -121,7 +123,9 @@ def migrate_etc(imgbase, new_lv, previous_lv):
             log.error("New: %s" % new_rel)
 
         if is_same_product:
-            idmaps = IDMap(old_etc, new_etc)
+            # The IDMap check must be run before etc was copied!
+            # The check relies on the fact that the old etc and new etc differ
+            idmaps = IDMap(old_etc, new_fs.path("/usr/share/factory/etc"))
             if idmaps.has_drift():
                 log.info("UID/GID drift was detected")
                 log.debug("Drifted uids: %s gids: %s" %
@@ -152,6 +156,49 @@ def migrate_etc(imgbase, new_lv, previous_lv):
         rsync = Rsync()
         rsync.sync(old_fs.path("/root/"), new_fs.path("/root"))
 
+        hack_rpm_permissions(new_fs)
+
+
+def hack_rpm_permissions(new_fs):
+    # FIXME changing the uid/gid is dropping the setuid.
+    # The following "solution" will use rpm to restore the
+    # correct permissions:
+    # rpm --setperms $(rpm --verify -qa | grep "^\.M\."
+    #                  | cut -d "/" -f2- | while read p ;
+    #                  do rpm -qf /$p ; done )
+    def just_do(arg, **kwargs):
+        DEVNULL = open(os.devnull, "w")
+        arg = ["nsenter", "--root=" + new_fs.path("/"), "--wd=/"] + arg
+        log.debug("Running %s" % arg)
+        proc = subprocess.Popen(arg, stdout=subprocess.PIPE,
+                                stderr=DEVNULL,
+                                **kwargs).communicate()
+        return proc[0]
+
+    incorrect_groups = {"paths": [],
+                        "verb": "--setugids"
+                        }
+    incorrect_paths = {"paths": [],
+                       "verb": "--setperms"
+                       }
+    for line in just_do(["rpm", "--verify", "-qa"]).splitlines():
+        _mode, _path = (line[0:13], line[13:])
+        if _mode[1] == "M":
+            incorrect_paths["paths"].append(_path)
+        if _mode[6] == "G":
+            incorrect_groups["paths"].append(_path)
+    log.debug("Incorrect groups according to rpm: %s" %
+              str(incorrect_groups["paths"]))
+    log.debug("Incorrect paths according to rpm: %s" %
+              str(incorrect_paths["paths"]))
+
+    for pgroup in [incorrect_groups, incorrect_paths]:
+        pkgs_req_update = just_do(["rpm", "-qf", "--queryformat",
+                                   "%{NAME}\n"] +
+                                  pgroup["paths"]).splitlines()
+        pkgs_req_update = list(set(pkgs_req_update))
+        just_do(["rpm", pgroup["verb"]] + pkgs_req_update)
+
 
 def adjust_mounts_and_boot(imgbase, new_lv, previous_lv):
     log.info("Inspecting if the layer contains OS data")
@@ -295,6 +342,9 @@ def adjust_mounts_and_boot(imgbase, new_lv, previous_lv):
         return kfiles
 
     def add_bootentry(newroot):
+        def _find_kfile(entry, kfiles):
+            return [f for f in kfiles if entry in f].pop()\
+                .replace("/boot", "").lstrip("/")
         if not File("%s/boot" % newroot).exists():
             log.info("New root does not contain a /boot, skipping.")
             return
@@ -335,10 +385,8 @@ def adjust_mounts_and_boot(imgbase, new_lv, previous_lv):
         kfiles = glob.glob(bootdir + "/*")
         # For the loader we are relative to /boot and need to
         # strip this part from the paths
-        bfile = lambda n: [f for f in kfiles if n in f].pop()\
-            .replace("/boot", "").lstrip("/")
-        vmlinuz = bfile("vmlinuz")
-        initrd = bfile("initramfs")
+        vmlinuz = _find_kfile("vmlinuz", kfiles)
+        initrd = _find_kfile("initramfs", kfiles)
         # FIXME default/grub cmdine and /etc/kernel… /var/kernel…
         grub_append = ShellVarFile("%s/etc/default/grub" % newroot)\
             .get("GRUB_CMDLINE_LINUX", "").strip('"').split()
